#include "api/df_access_control.h"
#include "datafox_library_iv.h"

DF_API_EXTERN_DECORATE int DF_API_CALL DFCEntrance2Identification( int connectionId, int deviceAddress, const char *readerTM, const char *identificationId, const char *identificationPin, char *statusText, int *statusTextLength, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false
        || readerTM == nullptr || identificationId == nullptr || identificationPin == nullptr || statusText == nullptr || statusTextLength == nullptr || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%s\t%s\t%s\t%s\t%i\t%i\t(%i, %i)", (readerTM == nullptr) ? "" : readerTM, (identificationId == nullptr) ? "" : identificationId,
            (identificationPin == nullptr) ? "" : identificationPin, (statusTextLength == nullptr) ? 0 : *statusTextLength, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );

        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );

        channelBridge->logEnd( "\t%s\t%s\t%s\t%s\t%i\t%i\t(%i, %i)", readerTM, identificationId, strlen(identificationPin) ? "*****" : "", statusText, *statusTextLength, *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->entrance2Identification( deviceAddress, readerTM, identificationId, identificationPin, statusText, (int&)*statusTextLength );
    if ( iRes == IV_ERROR_SUCCESS )
    {
        // Kein Fehler
        iRes = 1;
    }
    else if ( iRes > IV_ERROR_SUCCESS )
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ( (iRes >> 8) & 0xFF )
        {
            if ( ((iRes >> 8) & 0xFF) == 2 )
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_UNAVAILABLE );
            }
            else
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch ( iRes & 0xFF )
            {
            case 6:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_ENTRANCE2_OFF );
                break;

            case 28:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_NEED_PASSWORD );
                break;

            case 29:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PASSWORD_LOCK );
                break;

            default:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber( deviceAddress, -iRes );
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%s\t%s\t%s\t%s\t%i\t%i\t(%i, %i)", readerTM, identificationId, strlen(identificationPin) ? "*****" : "", statusText, *statusTextLength, *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCEntrance2OnlineAction( int connectionId, int deviceAddress, const char *readerTM, int mask, int type, int duration, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false
        || readerTM == nullptr || mask == 0 || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%s\t0x%08X\t%i\t%i\t%i\t(%i, %i)", (readerTM == nullptr) ? "" : readerTM, mask, type, duration, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );

        channelBridge->logEnd( "\t%s\t0x%08X\t%i\t%i\t%i\t(%i, %i)", readerTM, mask, type, duration, channelBridge->getLastErrorNumber( deviceAddress ), 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->entrance2OnlineAction( deviceAddress, readerTM, mask, type, duration );
    if ( iRes == IV_ERROR_SUCCESS )
    {
        // Kein Fehler
        iRes = 1;
    }
    else if ( iRes > IV_ERROR_SUCCESS )
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ( (iRes >> 8) & 0xFF )
        {
            if ( ((iRes >> 8) & 0xFF) == 2 )
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_UNAVAILABLE );
            }
            else
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch ( iRes & 0xFF )
            {
            case 6:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_ENTRANCE2_OFF );
                break;

            case 28:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_NEED_PASSWORD );
                break;

            case 29:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PASSWORD_LOCK );
                break;

            default:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber( deviceAddress, -iRes );
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%s\t0x%08X\t%i\t%i\t%i\t(%i, %i)", (readerTM == nullptr) ? "" : readerTM, mask, type, duration, *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCAccessControlIdentification( int connectionId, int deviceAddress, int masterId, const char *readerTM, const char *identificationId, const char *identificationPin, char *statusText, int *statusTextLength, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, masterId );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false
        || masterId < 1 || masterId > 9999
        || readerTM == nullptr || identificationId == nullptr || identificationPin == nullptr || statusText == nullptr || statusTextLength == nullptr || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%s\t%s\t%s\t%s\t%i\t%i\t(%i, %i)", (readerTM == nullptr) ? "" : readerTM, (identificationId == nullptr) ? "" : identificationId,
            (identificationPin == nullptr) ? "" : identificationPin, (statusTextLength == nullptr) ? 0 : *statusTextLength, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );

        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );

        channelBridge->logEnd( "\t%s\t%s\t%s\t%s\t%i\t%i\t(%i, %i)", readerTM, identificationId, strlen( identificationPin ) ? "*****" : "", statusText, *statusTextLength, *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->accessControlIdentification( deviceAddress, masterId, readerTM, identificationId, identificationPin, statusText, (int&)*statusTextLength );
    if ( iRes == IV_ERROR_SUCCESS )
    {
        // Kein Fehler
        iRes = 1;
    }
    else if ( iRes > IV_ERROR_SUCCESS )
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ( (iRes >> 8) & 0xFF )
        {
            if ( ((iRes >> 8) & 0xFF) == 2 )
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_UNAVAILABLE );
            }
            else
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch ( iRes & 0xFF )
            {
            case 6:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_ENTRANCE2_OFF );
                break;

            case 28:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_NEED_PASSWORD );
                break;

            case 29:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PASSWORD_LOCK );
                break;

            default:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber( deviceAddress, -iRes );
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%s\t%s\t%s\t%s\t%i\t%i\t(%i, %i)", readerTM, identificationId, strlen( identificationPin ) ? "*****" : "", statusText, *statusTextLength, *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCAccessControlOnlineAction( int connectionId, int deviceAddress, int masterId, const char *readerTM, int mask, int type, int duration, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, masterId );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress(deviceAddress) == false
        || masterId < 1 || masterId > 9999
        || readerTM == nullptr || mask == 0 || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%s\t0x%08X\t%i\t%i\t%i\t(%i, %i)", (readerTM == nullptr) ? "" : readerTM, mask, type, duration, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );

        channelBridge->logEnd( "\t%s\t0x%08X\t%i\t%i\t%i\t(%i, %i)", (readerTM == nullptr) ? "" : readerTM, mask, type, duration, *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->accessControlOnlineAction( deviceAddress, masterId, readerTM, mask, type, duration );
    if ( iRes == IV_ERROR_SUCCESS )
    {
        // Kein Fehler
        iRes = 1;
    }
    else if ( iRes > IV_ERROR_SUCCESS )
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ( (iRes >> 8) & 0xFF )
        {
            if ( ((iRes >> 8) & 0xFF) == 2 )
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_UNAVAILABLE );
            }
            else
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch ( iRes & 0xFF )
            {
            case 6:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_ENTRANCE2_OFF );
                break;

            case 28:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_NEED_PASSWORD );
                break;

            case 29:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PASSWORD_LOCK );
                break;

            default:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber( deviceAddress, -iRes );
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%s\t0x%08X\t%i\t%i\t%i\t(%i, %i)", (readerTM == nullptr) ? "" : readerTM, mask, type, duration, *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCAccessControlKnobCommand( int connectionId, int deviceAddress, int knobCommand, int masterId, int knobNumber, const char *params, char *buffer, int *bufferSize, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress ) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i\t%i\t%i\t%i", __func__, connectionId, deviceAddress, knobCommand, masterId, knobNumber );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false
        || (knobCommand != IV_VALUE_ACCESS_CONTROL_KNOB_COMMAND_TEACH && knobCommand != IV_VALUE_ACCESS_CONTROL_KNOB_COMMAND_DELETE && knobCommand != IV_VALUE_ACCESS_CONTROL_KNOB_COMMAND_LIST
        && knobCommand != IV_VALUE_ACCESS_CONTROL_KNOB_COMMAND_CHANGE_BATTERY_BEGIN && knobCommand != IV_VALUE_ACCESS_CONTROL_KNOB_COMMAND_CHANGE_BATTERY_END)
        || masterId < 1 || masterId > 9999
        || knobNumber < 0 || knobNumber > 15
        || (knobCommand == IV_VALUE_ACCESS_CONTROL_KNOB_COMMAND_LIST && (buffer == nullptr || bufferSize == nullptr))
        || errorNumber == nullptr )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        if ( errorNumber )
        {
            *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        }

        channelBridge->logEnd( "\t%s\t%s\t%i\t%i\t(%i, %i)", (params == nullptr) ? "" : params, (buffer == nullptr) ? "" : reinterpret_cast<char*>( buffer ), (bufferSize == nullptr) ? 0 : *bufferSize, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );

        channelBridge->logEnd( "\t%s\t%s\t%i\t%i\t(%i, %i)", (params == nullptr) ? "" : params, (buffer == nullptr) ? "" : reinterpret_cast<char*>( buffer ), (bufferSize == nullptr) ? 0 : *bufferSize, *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    int iRes = channelBridge->accessControlKnobCommand( deviceAddress, knobCommand, masterId, knobNumber, params, reinterpret_cast<unsigned char *>( buffer ), bufferSize );
    if ( iRes == IV_ERROR_SUCCESS )
    {
        // Kein Fehler
        iRes = 1;
    }
    else if ( iRes > IV_ERROR_SUCCESS )
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ( (iRes >> 8) & 0xFF )
        {
            if ( ((iRes >> 8) & 0xFF) == 2 )
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_UNAVAILABLE );
            }
            else
            {
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch ( iRes & 0xFF )
            {
            case 6:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_ENTRANCE2_OFF );
                break;

            case 28:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_NEED_PASSWORD );
                break;

            case 29:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PASSWORD_LOCK );
                break;

            default:
                channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_PROCESSING );
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber( deviceAddress, -iRes );
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        *errorNumber = channelBridge->getLastErrorNumber( deviceAddress );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%s\t%s\t%i\t%i\t(%i, %i)", (params == nullptr) ? "" : params, (buffer == nullptr) ? "" : reinterpret_cast<char*>( buffer ), (bufferSize == nullptr) ? 0 : *bufferSize, *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}
