#include "api/df_timeboy_lists.h"
#include "datafox_library_iv.h"

DF_API_EXTERN_DECORATE void DF_API_CALL DFCClearTimeboyListBuffer( int connectionId, int groupNumber, int number )
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
        return;

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, groupNumber, number);

    // zulaessige Bereiche pruefen
    if (groupNumber < -1 || groupNumber > 19 || number < -1 || number > 19)
    {
        channelBridge->setLastErrorNumber(0, IV_ERROR_FUNCTION_PARAMETER);
        channelBridge->logEnd("");
        return;
    }

    channelBridge->ClearTimeboyListBuffer(groupNumber, number);

    channelBridge->logEnd("");
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCMakeTimeboyList( int connectionId, int groupNumber, int number, int recordCount, int size, const unsigned char *buffer, int flags, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
    {
        if (errorNumber)
            *errorNumber = IV_ERROR_CHANNEL;

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i\t%i\t%i\t%i", __func__, connectionId, groupNumber, number, recordCount, size);

    // zulaessige Bereiche pruefen
    if (groupNumber < 0 || groupNumber > 19 || number < 0 || number > 19 || buffer == nullptr || errorNumber == nullptr)
    {
        channelBridge->setLastErrorNumber(0, IV_ERROR_FUNCTION_PARAMETER);

        if (errorNumber)
            *errorNumber = channelBridge->getLastErrorNumber(0);

        channelBridge->logEnd("\t\t%i\t%i\t(%i, %i)", flags, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber(0));
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    int iRes = channelBridge->MakeTimeboyList(groupNumber, number, recordCount, buffer, size, flags, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if (iRes == 0)
    {
        channelBridge->setLastErrorNumber(0, *errorNumber);
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(0));
    }

    channelBridge->logEnd("\t\t%i\t%i\t(%i, %i)", flags, *errorNumber, iRes, channelBridge->getLastErrorNumber(0));

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCLoadTimeboyList( int connectionId, int deviceAddress, int groupNumber, int number, int unused_setTo0, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
    {
        if (errorNumber)
            *errorNumber = IV_ERROR_CHANNEL;

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i\t%i\t%i\t%i", __func__, connectionId, deviceAddress, groupNumber, number, unused_setTo0);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false
        || groupNumber < -1 || groupNumber > 19 || number < -1 || number > 19
        || errorNumber == nullptr)
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        if (errorNumber)
            *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);

        channelBridge->logEnd("\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);

        channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    // Timeboy Listenuebertragung wird erst ab 04.01.xx unterstuetzt.
    unsigned char protocolVersion = channelBridge->getProtocolVersion(deviceAddress);
    if (protocolVersion == 0)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_UNREACHABLE);
        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);

        channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }
    else if (protocolVersion < 4)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_VERSION_FIRMWARE);
        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);

        channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    int iRes;
    std::string version;
    // Firmwareversion lesen
    if ((iRes = channelBridge->getVersionFirmware(deviceAddress, version)) != IV_ERROR_SUCCESS)
    {
        channelBridge->setLastErrorNumber(deviceAddress, (iRes < 0) ? -iRes : IV_ERROR_VERSION_FIRMWARE);
        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);

        channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }
    /*
    // Firmwareversion pruefen
    if (!theApp.m_util.CheckVersionFirmwareZK(version)) {

    *piError = IV_ERROR_FUNCTION_UNAVAILABLE;
    channelBridge->logEnd("\t%i\t(%i, %i)", *piError, 0);
    return 0;
    }
    */

    // Listendaten wegschreiben.
    if (channelBridge->logIsWriteList())
    {
        char fileNameBin[_MAX_PATH];
        char basePath[340];
        int listID, listNum;
        time_t t;
        tm *ltm;

        t = time(NULL);
        ltm = localtime(&t);

        for (listID = 0; listID < MAX_DATENSATZ_BESCHREIBUNG; ++listID)
        {
            for (listNum = 0; listNum < MAX_DATENSATZ_BESCHREIBUNG; ++listNum)
            {
                // Dateiname der Logdatei
                sprintf(fileNameBin, LOG_LIST_FILENAME_BIN, channelBridge->logGetBasePath(basePath),
                    ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday,
                    ltm->tm_hour, ltm->tm_min, ltm->tm_sec, connectionId, (char)(0x30 + listID), listNum);

                // Daten in die Datei schreiben.
                channelBridge->WriteListTimeboyToBinFile( listID, listNum, fileNameBin );
            }
        }
    }

    iRes = channelBridge->writeTimeboyListData(deviceAddress, groupNumber, number, unused_setTo0, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0)
    {
        channelBridge->setLastErrorNumber(deviceAddress, *errorNumber);
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));
    }

    channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber(deviceAddress));

    return iRes;
}
