#pragma once
#ifndef EXCHANGE_CLASSES_H
#define EXCHANGE_CLASSES_H

#include "exchange_structs.h"
#include <array>
#include <vector>

namespace Datafox
{
	
class FileHeader
{
public:
    typedef std::array<unsigned char, sizeof(StructFileHeader)> DataType;
    typedef std::vector<unsigned char> SerializeType;

public:
    FileHeader( EnumFileId id );
	
public:
    EnumFileId id() const;
    int size() const;
    StructFileHeader &data()
    {
        return reinterpret_cast<StructFileHeader &>( _structData );
    }

public:
    void resetData();
    void writeSerialize( SerializeType::iterator iter );
    void readSerialize( SerializeType::const_iterator iter );
    void readSerialize( SerializeType::const_iterator iter, const SerializeType &buffer );

protected:
    EnumFileId _id;
    DataType _structData;
};

class StructArray
{
public:
    typedef std::array<unsigned char, sizeof(StructStructArray)> DataType;
    typedef std::vector<unsigned char> SerializeType;

public:
    StructArray();

public:
    EnumStructId id() const;
    int size() const;
    StructStructArray &data();

public:
    void resetData();
    void writeSerialize( SerializeType::iterator iter );
    void readSerialize( SerializeType::const_iterator iter );
    void readSerialize( SerializeType::const_iterator iter, const SerializeType &buffer );

protected:
    EnumStructId _id;
    DataType _structData;
#ifdef _DEBUG
    StructStructArray *_data;
#endif

public:
    std::string toJson();
};

/*
template<class StructExchange> class ExchangeBase
{
public:
    typedef std::array<unsigned char, sizeof(StructExchange)> DataType;
    typedef std::vector<unsigned char> SerializeType;
    
public:
	ExchangeBase( StructId id );
    
public:
    const StructId id() const;
    const int size() const;
    StructExchange &data();

public:
    void reset();
    void writeSerialize( SerializeType::iterator iter );
    void readSerialize( SerializeType::const_iterator iter );
    void readSerialize( SerializeType::const_iterator iter, const SerializeType &buffer );

protected:
    StructId _id;
    DataType _structData;
};
*/
}

#endif
