/**
* @file  df_log.h
* @brief Schnittstelle fuer die Klasse CDFLog.
*/
//////////////////////////////////////////////////////////////////////
#pragma once
#ifndef DF_LOG_H
#define DF_LOG_H

#include <ctime>
#include <string>

// Initialisierungsdatei verwalten
#include "file/file_ini.h"
// Hilfsklasse zur Verwaltung alter Logdateien
#include "df_log_check.h"

#include <mutex>
#include <functional>

/* Die Logklasse ist fuer ein standardmaessiges logen von Funktionsaufrufen,
* so dass Fehler in der Verarbeitung auch noch nach einigen Tagen 
* nachfolzogen werden koennten.
* Sie ist ueber eine Initialisierungsdatei mit grundinformationen versorgbar.
*
*/
#define SUB_FOLDER           "DFComLog"

#define INI_FILENAME         "DFCom.ini"
// YYYY_MM_DD_DFComDLL_channelID.log
#define LOG_FILENAME         "%s/%s/20%02i_%02i_%02i_DFComDLL_%i.log"
#define LOG_SEARCH_FILENAME  "%s/%s/20??_??_??_DFComDLL_*.log"
#define LOG_DELETE_FILENAME  "%s/%s/%s"

#if defined(_MSC_VER) || defined(__MINGW32__)
#define fullpath             _fullpath
#define mkdir                _mkdir
#define stat                 _stat
#define finddata_t           _finddata_t
#define findfirst            _findfirst
#define findnext             _findnext
#define findclose            _findclose
#define snprintf	         _snprintf
#define access               _access
#endif

class CDFLog
{
    bool m_enabled;                    // Log aktiv?
    std::string m_file_path;           // Dateiname der Logdatei
    bool m_erase_files;                // Alte Dateien loeschen
    bool m_elapse;                     // Ausfuehrungszeit angeben
    int m_write_list;                  // Listendaten mitloggen
public:
    int m_docking_prefer_type;         // Bevorzugte Version der Dockingstation
    int m_trace_mode;                  // Tracing aktivieren
    bool m_check_open_and_close_by_same_thread; // Das Oeffnen und Schliessen des Kanals muss durch den selben Thread erfolgen.

public:
    CDFLog( int connection_id, std::function<void( const std::string& )> change_trace_file_path );
    virtual ~CDFLog();

    // Ist Log aktiv?
    inline bool logEnabled()
    {
        return m_enabled;
    }

    // Erzeugen einer Pfadangabe
    void MakeFilePath( char *path_buffer, char *szFileName, char *szFileExt );

    // Finden einer ggf. vorhandenen Initialisierungsdatei.
    std::string find_ini_file( bool create_if_not_exist = true );
    
    // Erstellen einer Inidatei mit Standardwerten.
    bool create_ini_file( const std::string &file_path, const std::string &log_base_path );

    // Einlesen der Initialisierungsdatei
    void logIniRead();
    // Pruefen auf alte Dateien.
    void logCheck();

    // Schreiben einer Logzeile, wenn Log aktiv.
    void logBegin( const char *Format, ... ); // Mit Uhrzeit
    void logAppend( const char *szFormat, ... );
    void logEnd( const char *szFormat, ... ); // Mit Zeilenumbruch
    char *logGetBasePath( char *basePath )
    {
        std::sprintf( basePath, "%s%s", m_log_check.m_base_path, m_log_check.m_sub_folder );
        return basePath;
    }
    int logIsWriteList() { return m_enabled ? m_write_list : 0; }

    std::function<void (const std::string & )> cb_change_trace_file_path;
    std::string get_current_log_file_path();
    std::string get_current_trace_file_path();
    void check_current_file_paths();

protected:
    // Zugewiesene Kanalnummer
    int m_connection_id;
    // Zu verwendende Initialisierungsdatei
    FileIni m_log_ini;
    // Start des Logvorganges
    std::clock_t m_clock_begin;
    // Alte Logdateien verwalten
    CDFLogCheck m_log_check;
    // Bereichssperre fuer Dateizugriff.
    std::mutex m_mutex_file;
};

#endif

